/* 
 * File:   rf.h
 * Author: Crazy Cat
 *
 * Created on 21  2015 ., 12:37
 */

#ifndef RF_H
#define	RF_H

#include <stdint.h>

#define RF_SPI_CSN(val) RF_CSN_PIN = val
#define RF_CE(val) RF_CE_PIN = val

// rf commands
#define R_REGISTER         0x1FU  /**< Register write command */
#define W_REGISTER         0x20U  /**< Register write command */
#define R_RX_PAYLOAD       0x61U  /**< Read RX payload command */
#define W_TX_PAYLOAD       0xA0U  /**< Write TX payload command */
#define FLUSH_TX           0xE1U  /**< Flush TX register command */
#define FLUSH_RX           0xE2U  /**< Flush RX register command */
#define REUSE_TX_PL        0xE3U  /**< Reuse TX payload command */
#define ACTIVATE           0x50U  /**< Activate features */
#define R_RX_PL_WID        0x60U  /**< Read RX payload command */
#define W_ACK_PAYLOAD      0xA8U  /**< Write ACK payload command */
#define W_TX_PAYLOAD_NOACK 0xB0U  /**< Write ACK payload command */
#define NOOP               0xFFU  /**< No Operation command, used for reading status register */


//***************************************************//
/** @name  - Register Memory Map - */
#define CONFIG_REG          0x00  // 'Config' register address
#define EN_AA_REG           0x01  // 'Enable Auto Acknowledgment' register address
#define EN_RXADDR_REG       0x02  // 'Enabled RX addresses' register address
#define SETUP_AW_REG        0x03  // 'Setup address width' register address
#define SETUP_RETR_REG      0x04  // 'Setup Auto. Retrans' register address
#define RF_CH_REG           0x05  // 'RF channel' register address
#define RF_SETUP_REG        0x06  // 'RF setup' register address
#define STATUS_REG          0x07  // 'Status' register address
#define OBSERVE_TX_REG      0x08  // 'Observe TX' register address
#define CD_REG              0x09  // 'Carrier Detect' register address              NRF24L01+
#define RPD_REG             0x09  // 'Received Power Detector' register address     NRF24L01, SE8R01???
#define RX_ADDR_P0_REG      0x0A  // 'RX address pipe0' register address
#define RX_ADDR_P1_REG      0x0B  // 'RX address pipe1' register address
#define RX_ADDR_P2_REG      0x0C  // 'RX address pipe2' register address
#define RX_ADDR_P3_REG      0x0D  // 'RX address pipe3' register address
#define RX_ADDR_P4_REG      0x0E  // 'RX address pipe4' register address
#define RX_ADDR_P5_REG      0x0F  // 'RX address pipe5' register address
#define TX_ADDR_REG         0x10  // 'TX address' register address
#define RX_PW_P0_REG        0x11  // 'RX payload width, pipe0' register address
#define RX_PW_P1_REG        0x12  // 'RX payload width, pipe1' register address
#define RX_PW_P2_REG        0x13  // 'RX payload width, pipe2' register address
#define RX_PW_P3_REG        0x14  // 'RX payload width, pipe3' register address
#define RX_PW_P4_REG        0x15  // 'RX payload width, pipe4' register address
#define RX_PW_P5_REG        0x16  // 'RX payload width, pipe5' register address
#define FIFO_STATUS_REG     0x17  // 'FIFO Status Register' register address
#define DYNPD_REG           0x1CU /**< nRF24L01 Dynamic payload setup */
#define FEATURE_REG         0x1DU /**< nRF24L01 Exclusive feature setup */



//se8r01 only
#define STA_MARK_RX         0X40
#define STA_MARK_TX         0X20
#define STA_MARK_MX         0X10

///////////////////////////////////////////////////////////////////////////////////////
#define iBANK0                  0x00
#define iBANK1                  0x80
//
//-------------------------------------------------------------------------------


//-------------------------------------------------------------------------------
// SE8R01 registers addresses
//
#define iRF_BANK0_SETUP_VALUE   0x1E
#define iRF_BANK0_PRE_GURD      0x1F

//SE8R01 Bank1 register
#define iRF_BANK1_LINE          0x00
#define iRF_BANK1_PLL_CTL0      0x01
#define iRF_BANK1_PLL_CTL1      0x02
#define iRF_BANK1_CAL_CTL       0x03
#define iRF_BANK1_A_CNT_REG     0x04
#define iRF_BANK1_B_CNT_REG     0x05
#define iRF_BANK1_RESERVED0     0x06
#define iRF_BANK1_STATUS        0x07
#define iRF_BANK1_STATE         0x08
#define iRF_BANK1_CHAN          0x09
#define iRF_BANK1_IF_FREQ       0x0A
#define iRF_BANK1_AFC_COR       0x0B
#define iRF_BANK1_FDEV          0x0C
#define iRF_BANK1_DAC_RANGE     0x0D
#define iRF_BANK1_DAC_IN        0x0E
#define iRF_BANK1_CTUNING       0x0F
#define iRF_BANK1_FTUNING       0x10
#define iRF_BANK1_RX_CTRL       0x11
#define iRF_BANK1_FAGC_CTRL     0x12
#define iRF_BANK1_FAGC_CTRL_1   0x13
#define iRF_BANK1_DAC_CAL_LOW   0x17
#define iRF_BANK1_DAC_CAL_HI    0x18
#define iRF_BANK1_RESERVED1     0x19
#define iRF_BANK1_DOC_DACI      0x1A
#define iRF_BANK1_DOC_DACQ      0x1B
#define iRF_BANK1_AGC_CTRL      0x1C
#define iRF_BANK1_AGC_GAIN      0x1D
#define iRF_BANK1_RF_IVGEN      0x1E
#define iRF_BANK1_TEST_PKDET    0x1F
//
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// SE8R01 interrupt status
//
#define iSTATUS_RX_DR           0x40
#define iSTATUS_TX_DS           0x20
#define iSTATUS_MAX_RT          0x10
#define iSTATUS_TX_FULL         0x01
//
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// SE8R01 FIFO status
//
#define iFIFO_STATUS_TX_REUSE   0x40
#define iFIFO_STATUS_TX_FULL    0x20
#define iFIFO_STATUS_TX_EMPTY   0x10

#define iFIFO_STATUS_RX_FULL    0x02
#define iFIFO_STATUS_RX_EMPTY   0x01
////////////////////////////////////////////////

uint8_t SPI_RW_Reg(const uint8_t reg, const uint8_t value);
uint8_t SPI_Write_Buf(const uint8_t reg, const uint8_t *pBuf, uint8_t bytes);
uint8_t SPI_Read_Buf(const uint8_t reg, uint8_t *pBuf, const uint8_t bytes);

uint8_t SPI_Write_Buf1(const uint8_t reg, uint8_t bytes, ...);

void se8r01_powerup();
void se8r01_calibration();
void se8r01_setup();

#endif	/* RF_H */

